@@ -54,7 +54,7 @@ const char* get_optional_features(void);
 
 cJSON* iperf_json_printf(const char *format, ...);
 
-void iperf_dump_fdset(FILE *fp, char *str, int nfds, fd_set *fds);
+void iperf_dump_fdset(FILE *fp, const char *str, int nfds, fd_set *fds);
 
 #ifndef HAVE_DAEMON
 extern int daemon(int nochdir, int noclose);
